import axios from "axios";

import {ElMessage} from 'element-plus'
import {Loading} from "@/helper/loading.js";

const http = axios.create({
    baseURL: import.meta.env.VITE_API_BASE_URL,
    timeout: 60 * 1000,
})

http.defaults.headers.post['Content-Type'] = "application/json"



const loadingObj = new Loading()


http.interceptors.request.use(
    (config) => {
        loadingObj.addLoading()
        // console.log(config)

        let user = import.meta.env.VITE_WEB_USER
        let password = import.meta.env.VITE_WEB_PASSWORD
        if (user && password) {
            config.auth = {
                username: user,
                password: password
            }
        }

        return config
        // return new Promise((resolve, reject) => {
        //     resolve(config)
        // })
    },
    err => {
        return Promise.reject(err)
    },
)

http.interceptors.response.use(
    res => {
        loadingObj.closeLoading()
        if (res.data.code !== 0) {
            ElMessage({
                message: res.data.message,
                type: 'error',
                showClose: true,
            })
            return Promise.reject(res)
        }
        //
        // ElMessage({
        //     message: 'Congrats, this is a success message.',
        //     type: 'success',
        // })

        return res
    },

    err => {
        loadingObj.closeLoading()

        ElMessage({
            message: err,
            type: 'error',
            showClose: true,
        })

        return Promise.reject(err)

    }
)


export default http